#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

extras_require = \
{'dev': ['pytest', 'pytest-cov', 'wheel', 'coveralls']}

setup(name='aiodec',
      version='2018.10.1',
      description='aiodec',
      author='Caleb Hattingh',
      author_email='caleb.hattingh@gmail.com',
      url='https://github.com/cjrh/aiodec',
      py_modules=['aiodec'],
      extras_require=extras_require,
     )
